using System.Reflection;
using Asp.Versioning.ApiExplorer;
using LuskiServer;
using LuskiServer.Classes;
using LuskiServer.Classes.TableDef;
using LuskiServer.Enums;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;
using ServerDatabase;
using Swashbuckle.AspNetCore.SwaggerGen;

[assembly: Microsoft.AspNetCore.Mvc.ApiController]

Luski.Config = Luski.GetSettings("/etc/luskiserver/app.json", AppConfigContext.Default.AppConfig, true);
Console.WriteLine($"Server starting with starting epoch of {DateTime.UnixEpoch.AddMilliseconds(Luski.Config.ServerEpoch).ToLocalTime()}");
Luski.Database = new Database(Luski.Config.Address, 
    Luski.Config.Database, 
    Luski.Config.Username, 
    Luski.Config.Password,
    Luski.Config.CustomeName);

try
{
    Luski.Database.ExecuteNonQuery($"CREATE DATABASE {Luski.Config.Database} WITH OWNER = {Luski.Config.Username} ENCODING = 'UTF8' CONNECTION LIMIT = -1 IS_TEMPLATE = False;");
    Console.WriteLine("Database Created");
}
catch (Exception e)
{
    Console.WriteLine("Database Found");
}

foreach (PropertyInfo prop in typeof(Tables).GetProperties())
{
    Table table = (Table)prop.GetValue(null)!;
    table.DatabaseHandler = Luski.Database;
    Type t = Type.GetType($"LuskiServer.Classes.TableDef.{prop.Name}")!;
    foreach(PropertyInfo prop2 in t.GetProperties())
    {
        table.WithColumn((ITableColumn)prop2.GetValue(null)!);
    }
    Luski.Database.Tables.Add(table);
}

Luski.Database.RegisterTables();

if (!Luski.Database.VersionsTable.TryRead(Luski.Database.VersionsTable.ID, out _,
        Luski.Database.VersionsTable.ID.CreateParameter(0)))
{
    Luski.Database.VersionsTable.Insert(Luski.Database.VersionsTable.ID.CreateParameter(0));
}

if (!Tables.Roles.TryRead(Roles.ID, out _, Roles.ID.CreateParameter(0)))
{
    Tables.Roles.Insert(
        Roles.ID.CreateParameter(0),
        Roles.Name.CreateParameter("server"),
        Roles.Color.CreateParameter("5,5,5,5"),
        Roles.Description.CreateParameter("The default role for the server. Everybody will have this role."),
        Roles.ServerPermissions.CreateParameter(new[]
        {
            ServerPermissions.ViewChannels,
            ServerPermissions.ViewCategories,
            ServerPermissions.Nickname,
            ServerPermissions.SendMessages,
            ServerPermissions.SendFiles,
            ServerPermissions.ChannelAndServerPings,
            ServerPermissions.PingSomeone,
            ServerPermissions.ReadMessageHistory,
            ServerPermissions.UseServerCommands,
            ServerPermissions.JoinVoice,
            ServerPermissions.SpeakInVoice
        })
    );
}

if (!Tables.ServerRoleOverides.TryRead(ServerRoleOverides.ID, out _, Categories.ID.CreateParameter(0)))
{
    Tables.ServerRoleOverides.Insert(
        ServerRoleOverides.ID.CreateParameter(0),
        ServerRoleOverides.RoleID.CreateParameter(0),
        ServerRoleOverides.Overides.CreateParameter(new string[]
        {
            $"{ServerPermissions.ViewCategories.GetNumberString()}:1",
            $"{ServerPermissions.ViewChannels.GetNumberString()}:1",
        })
    );
}

if (!Tables.Categories.TryRead(Categories.ID, out _, Categories.ID.CreateParameter(0)))
{
    Tables.Categories.Insert(
        Categories.ID.CreateParameter(0),
        Categories.Name.CreateParameter("server"),
        Categories.Description.CreateParameter(
            "The default category for the server. Everybody will see this category."),
        Categories.RoleOverides.CreateParameter(new long[1] { 0 })
    );
}



var builder = WebApplication.CreateBuilder( args );

// Add services to the container.

builder.Services.AddControllers();
builder.Services.AddProblemDetails();

builder.Services.AddApiVersioning(
                    options =>
                    {
                        // reporting api versions will return the headers
                        // "api-supported-versions" and "api-deprecated-versions"
                        options.ReportApiVersions = true;
                        /*    Sunset example
                        options.Policies.Sunset( 1 )
                                        //.Effective( DateTimeOffset.Now.Subtract( new TimeSpan(9999999) ) )
                                        .Effective( DateTimeOffset.Now.AddDays( 2 ) )
                                        .Link( "policy.html" )
                                            .Title( "Versioning Policy" )
                                            .Type( "text/html" );
                                            */
                    } )
                .AddMvc()
                .AddApiExplorer(
                    options =>
                    {
                        // add the versioned api explorer, which also adds IApiVersionDescriptionProvider service
                        // note: the specified format code will format the version as "'v'major[.minor][-status]"
                        options.GroupNameFormat = "'v'VVV";

                        // note: this option is only necessary when versioning by url segment. the SubstitutionFormat
                        // can also be used to control the format of the API version in route templates
                        options.SubstituteApiVersionInUrl = true;
                    } );
builder.Services.Configure<ApiBehaviorOptions>(options =>
{
    options.InvalidModelStateResponseFactory = actionContext => 
    {
        string content;
        using (StreamReader reader = new(actionContext.HttpContext.Request.Body))
        {
            content = reader.ReadToEndAsync().Result;
        }

        if (string.IsNullOrEmpty(content)) return new ObjectResult(new HTTPResponse()
        {
            error = ErrorCode.MissingPostData,
            error_message = "The post information was missing"
        })
        {
            StatusCode = 403
        };
        return new ObjectResult(new HTTPResponse()
        {
            error = ErrorCode.InvalidPostData,
            error_message = "The provided post information was invalid"
        })
        {
            StatusCode = 403
        };
    };
});
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddTransient<IConfigureOptions<SwaggerGenOptions>, ConfigureSwaggerOptions>();
builder.Services.AddSwaggerGen(
    options =>
    {
        // add a custom operation filter which sets default values
        options.OperationFilter<SwaggerDefaultValues>();

        var fileName = typeof(Program).Assembly.GetName().Name + ".xml";
        var filePath = Path.Combine(AppContext.BaseDirectory, fileName);

        // integrate xml comments
        options.IncludeXmlComments(filePath);
    } );

var app = builder.Build();

// Configure the HTTP request pipeline.

app.UseSwagger();
app.UseSwaggerUI(
    options =>
    {
        IReadOnlyList<ApiVersionDescription> descriptions = app.DescribeApiVersions();

        // build a swagger endpoint for each discovered API version
        foreach ( var description in descriptions )
        {
            string url = $"/swagger/{description.GroupName}/swagger.json";
            string name = description.GroupName.ToUpperInvariant();
            options.SwaggerEndpoint(url, name);
        }
        options.InjectStylesheet("/www/css/SwaggerDark.css");
        options.DocumentTitle = "Luski Server API";
    } );
app.MapGet("/www/css/SwaggerDark.css", async (CancellationToken t) =>
{
    var css = await File.ReadAllBytesAsync("www/css/SwaggerDark.css", t);
    return Results.File(css, "text/css");
}).ExcludeFromDescription();
app.UseHttpsRedirection();
app.UseAuthorization();
app.MapControllers();
app.Run();